/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2014 Red Hat, Inc., and individual contributors
 * as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jboss.modules.security;

import java.security.Permission;

/**
 * A permission factory which simply holds a permission object.
 *
 * @author <a href="mailto:david.lloyd@redhat.com">David M. Lloyd</a>
 */
public final class ImmediatePermissionFactory implements PermissionFactory {
    private final Permission permission;

    /**
     * Construct a new instance.
     *
     * @param permission the permission to return
     */
    public ImmediatePermissionFactory(final Permission permission) {
        if (permission == null) {
            throw new IllegalArgumentException("permission is null");
        }
        this.permission = permission;
    }

    public Permission construct() {
        return permission;
    }
}
