/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2016 Red Hat, Inc., and individual contributors
 * as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jboss.modules;

import java.util.Iterator;
import java.util.function.Function;

/**
 * @author <a href="mailto:david.lloyd@redhat.com">David M. Lloyd</a>
 */
final class IteratorUtils {
    private IteratorUtils() {
    }

    public static <T, U> Iterator<U> transformingIterator(Iterator<T> original, Function<T, U> translator) {
        return new Iterator<U>() {
            public boolean hasNext() {
                return original.hasNext();
            }

            public U next() {
                return translator.apply(original.next());
            }
        };
    }
}
